using System;
using System.Collections.Generic;
using System.Text;

namespace AdvancedEvent
{
    delegate void NameChangedDelegate(string name, string newValue);

    class Program
    {
        static void Main(string[] args)
        {
            Customer c = new Customer();
            Subscriber s1 = new Subscriber(c, "subscriber-A");
            Subscriber s2 = new Subscriber(c, "subscriber-B");
            Subscriber s3 = new Subscriber(c, "subscriber-C");

            c.FirstName = "Kevin";
            c.LastName = "Hoffman";

            s2.Unsubscribe();

            c.FirstName = "Joe";

            c.MiddleName = "Scott";

            Console.ReadLine();
        }

        
    }

    class Subscriber
    {
        private string subscriberId = "new subscriber";
        private Customer myCustomer = null;
        private NameChangedDelegate ncDel = null;

        public Subscriber(Customer c, string subId)
        {
            subscriberId = subId;
            ncDel = new NameChangedDelegate(myCustomer_OnNameChanged);
            myCustomer = c;
            myCustomer.OnNameChanged += ncDel;
        }

        void myCustomer_OnNameChanged(string name, string newValue)
        {
            Console.WriteLine("[{0}] Customer {1} changed to {2}.", subscriberId, 
                name, newValue);
        }

        public void Unsubscribe()
        {
            myCustomer.OnNameChanged -= ncDel;
        }

        public string SubscriberID
        {
            get
            {
                return subscriberId;
            }
            set
            {
                subscriberId = value;
            }
        }
    }

    class Customer
    {
        private string firstName;
        private string lastName;
        private string middleName;
        private event NameChangedDelegate onNameChange;

        public event NameChangedDelegate OnNameChanged
        {
            add
            {
                onNameChange += value;
                if (value.Target is Subscriber)
                {
                    Console.WriteLine("Subscriber '{0}' just subscribed to OnNameChanged.",
                        ((Subscriber)value.Target).SubscriberID);
                }
            }
            remove
            {
                onNameChange -= value;
                if (value.Target is Subscriber)
                {
                    Console.WriteLine("Subscriber '{0}' just un-subscribed from OnNameChanged.",
                        ((Subscriber)value.Target).SubscriberID);
                }
            }
        }

        public string FirstName
        {
            get { return firstName; }
            set
            {
                firstName = value;
                onNameChange("firstname", value);
            }
        }

        public string LastName
        {
            get { return lastName; }
            set
            {
                lastName = value;
                onNameChange("lastname", value);
            }
        }

        public string MiddleName
        {
            get { return MiddleName; }
            set
            {
                middleName = value;
                NotifyAllBut("subscriber-C", "middlename", value);
            }
        }

        private void NotifyAllBut(string subId, string name, string value)
        {
            foreach (NameChangedDelegate d in onNameChange.GetInvocationList())
            {
                if (((Subscriber)d.Target).SubscriberID.ToUpper() != subId.ToUpper())
                {
                    d(name, value);
                }
            }
        }
    }
}
