using System;
using System.Collections.Generic;
using System.Text;

namespace MulticastDemo
{
    class Program
    {
        delegate void MessagePrintDelegate(string s);

        static void Main(string[] args)
        {
            MessagePrintDelegate mpd1, mpd2, mpd3, mpd4;

            mpd1 = PrintMessage;
            mpd2 = PrintStampedMessage;

            // create a multicast by composing two delegates
            mpd3 = mpd1 + mpd2;
            mpd3("Hello World");

            mpd4 = PrintMessage;
            mpd4 += mpd3;
            mpd4("Multicasted!");

            // you can also remove a delegate from a composition
            mpd3 -= mpd1;
            mpd3("Hello World (again)");

            // you can compose 2 multicasts
            mpd4 += mpd3;
            mpd4("Really multicasted");

            // you can even add an anonymous method to the multicast
            mpd3 += delegate(string s)
            {
                Console.WriteLine("[Anonymous] {0}", s);
            };
            mpd3("Hello!");

            // you can inspect the list of methods
            // in any given multicast
            Console.WriteLine("Methods in the mpd3 multicast:");
            foreach (Delegate d in mpd3.GetInvocationList())
            {
                Console.WriteLine("\t{0}", d.Method.Name);
            }

            Console.ReadLine();
        }

        static void PrintMessage(string s)
        {
            Console.WriteLine(s);
        }

        static void PrintStampedMessage(string s)
        {
            Console.WriteLine("[{0}] {1}", DateTime.Now.ToShortTimeString(),
                s);
        }
    }
}
