using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Deployment.Application;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace ClickOnce1
{
    public partial class Form1 : Form
    {
        private ApplicationDeployment ad = null;

        public Form1()
        {
            InitializeComponent();
            ad = ApplicationDeployment.CurrentDeployment;
            ad.CheckForUpdateCompleted += 
                new CheckForUpdateCompletedEventHandler(ad_CheckForUpdateCompleted);
            ad.UpdateProgressChanged += 
                new DeploymentProgressChangedEventHandler(ad_UpdateProgressChanged);
            ad.CheckForUpdateProgressChanged += 
                new DeploymentProgressChangedEventHandler(ad_CheckForUpdateProgressChanged);
            ad.UpdateCompleted += new AsyncCompletedEventHandler(ad_UpdateCompleted);
            this.Text = "ClickOnce Application (v" + ad.CurrentVersion.ToString() + ")";
        }

        void ad_UpdateProgressChanged(object sender, DeploymentProgressChangedEventArgs e)
        {
            tsProgress.Value = e.ProgressPercentage;
        }

        void ad_CheckForUpdateProgressChanged(object sender, DeploymentProgressChangedEventArgs e)
        {
            tsProgress.Value = e.ProgressPercentage;
        }

        void ad_UpdateCompleted(object sender, AsyncCompletedEventArgs e)
        {
            if (e.Error != null)
            {
                MessageBox.Show("There was an error updating the application:\n" + 
                    e.Error.ToString());
                return;
            }

            if (MessageBox.Show("Update installed successfully. Restart application?", 
                "Application Restart",
                MessageBoxButtons.YesNo) == DialogResult.Yes)
            {
                Application.Restart();
            }
        }

        void ad_CheckForUpdateCompleted(object sender, CheckForUpdateCompletedEventArgs e)
        {
            if (e.Error != null)
            {
                MessageBox.Show("Failed to check for update: \n" + e.ToString());
                return;
            }
            if (e.UpdateAvailable)
            {
                tsLabel.Text = "Update Found!";
                tsProgress.Value = 0;
                if (e.IsUpdateRequired)
                {
                    MessageBox.Show("A mandatory update is available. Update will begin immediately.");                    
                    ad.UpdateAsync();
                }
                else
                {
                    if (MessageBox.Show("An optional update is available. Would you like to download it now?",
                        "Update Available", MessageBoxButtons.YesNo) == DialogResult.Yes)
                    {
                        ad.UpdateAsync();
                    }
                }
            }
            else
            {
                tsLabel.Text = "ClickOnce Application";
                tsProgress.Value = 0;
                MessageBox.Show("No new updates are available for this application.");
            }
           
        }

        private void aboutToolStripMenuItem_Click(object sender, EventArgs e)
        {
            FormLibrary.AboutBox1 aBox = new FormLibrary.AboutBox1();
            aBox.ShowDialog();
        }

        private void updateNowToolStripMenuItem_Click(object sender, EventArgs e)
        {
            // if this application is a clickonce app
            if (ApplicationDeployment.IsNetworkDeployed)
            {
                ad.CheckForUpdateAsync();
                tsLabel.Text = "Checking for Updates...";
                tsProgress.Value = 0;
            }
        }
    }
}